import unittest
from test.test_support import EnvironmentVarGuard
from outlyer.plugin_helper.container import crequests

CONTAINER_ID = 'container_12345'
CONTAINER_IP = '123.45.667.890'


class TestDockerHelpers(unittest.TestCase):

    def setUp(self):
        self.env = EnvironmentVarGuard()

    def test_sanitise_endpoint(self):
        self.env.set('CONTAINER_IP', CONTAINER_IP)
        self.env.set('CONTAINER_ID', CONTAINER_ID)
        se = crequests.sanitize_container_endpoint("http://localhost/blah")
        self.assertEqual('http://%s/blah' % CONTAINER_IP, se)

    def test_sanitise_endpoint_port(self):
        self.env.set('CONTAINER_IP', CONTAINER_IP)
        self.env.set('CONTAINER_ID', CONTAINER_ID)
        se = crequests.sanitize_container_endpoint("http://localhost:43/blah")
        self.assertEqual('http://%s:43/blah' % CONTAINER_IP, se)

    def test_sanitise_endpoint_not_docker(self):
        self.env.unset('CONTAINER_ID')
        self.env.set('CONTAINER_IP', CONTAINER_IP)
        ep = "http://localhost:43/blah"
        se = crequests.sanitize_container_endpoint(ep)
        self.assertEqual(ep, se)

    def test_sanitise_endpoint_no_container_ip(self):
        self.env.unset('CONTAINER_IP')
        self.env.set('CONTAINER_ID', CONTAINER_ID)
        ep = "http://localhost:43/blah"
        se = crequests.sanitize_container_endpoint(ep)
        self.assertEqual(ep, se)


import os
import subprocess
import docker


def check_output(cmd):
    if is_container():
        cid = get_container_id()
        client = docker.from_env()
        target = client.containers.get(cid)
        return target.exec_run(cmd)

    elif is_patched():
        return subprocess._check_output(cmd)

    else:
        return subprocess.check_output(cmd)


def patch():
    if not is_patched():
        subprocess._check_output = subprocess.check_output
        subprocess.check_output = check_output


def unpatch():
    if is_patched():
        subprocess.check_output = subprocess._check_output
        delattr(subprocess, '_check_output')


def is_patched():
    return hasattr(subprocess, '_check_output')


def is_container():
    if get_container_id():
        return True
    else:
        return False


def get_container_ip():
    return os.environ.get('CONTAINER_IP')


def get_container_id():
    return os.environ.get("CONTAINER_ID")



import urlparse

import requests

from outlyer.plugin_helper import container


def get(endpoint, **kwargs):
    return request('get')


def put(endpoint, **kwargs):
    return request('put')


def post(endpoint, **kwargs):
    return request('post')


def delete(endpoint, **kwargs):
    return request('delete')


def patch(endpoint, **kwargs):
    return request('patch')


def head(endpoint, **kwargs):
    return request('head')


def request(method, endpoint, **kwargs):
    endpoint = sanitize_container_endpoint(endpoint)
    return requests.request(method, endpoint, **kwargs)


def sanitize_container_endpoint(endpoint):
    if not container.is_container():
        return endpoint

    parsed = urlparse.urlparse(endpoint)
    if parsed.hostname in ['127.0.0.1', 'localhost']:
        endpoint = _set_to_container_ip(parsed)

    return endpoint


def _set_to_container_ip(parse_endpoint):
    parts = list(parse_endpoint)
    hostname = container.get_container_ip()

    if not hostname:
        hostname = parse_endpoint.hostname

    if parse_endpoint.port:
        hostname = "%s:%s" % (hostname, parse_endpoint.port)

    parts[1] = hostname

    return urlparse.urlunparse(parts)




import subprocess
import unittest
from mock import Mock
from test.test_support import EnvironmentVarGuard
from outlyer.plugin_helper import container

CONTAINER_ID = 'container_12345'
CONTAINER_IP = '123.45.667.890'


class TestDockerHelpers(unittest.TestCase):

    def setUp(self):
        self.env = EnvironmentVarGuard()

    def test_is_container_when_container_id(self):
        self.env.set('CONTAINER_ID', CONTAINER_ID)
        self.assertTrue(container.is_container())

    def test_not_container_when_no_container_id(self):
        self.env.unset('CONTAINER_ID')
        self.assertFalse(container.is_container())

    def test_conatiner_id(self):
        self.env.set('CONTAINER_ID', CONTAINER_ID)
        self.assertEqual(CONTAINER_ID, container.get_container_id())

    def test_conatiner_ip(self):
        self.env.set('CONTAINER_IP', CONTAINER_IP)
        self.assertEqual(CONTAINER_IP, container.get_container_ip())

    def test_patch_container(self):
        container.patch()
        self.assertTrue(hasattr(subprocess, '_check_output'))

    def test_unpatch_container(self):
        container.unpatch()
        self.assertFalse(hasattr(subprocess, '_check_output'))

    def test_check_output_no_container_patched(self):
        self.env.unset('CONTAINER_ID')
        container.patch()
        subprocess._check_output = Mock()
        subprocess.check_output = Mock()
        container.check_output(['ls', '/tmp'])
        subprocess._check_output.assert_called_with(['ls', '/tmp'])
        subprocess.check_output.assert_not_called()

    def test_check_output_no_container_not_patched(self):
        self.env.unset('CONTAINER_ID')
        container.unpatch()
        subprocess.check_output = Mock()
        container.check_output(['ls', '/tmp'])
        subprocess.check_output.assert_called_with(['ls', '/tmp'])




from setuptools import setup


setup(
    name='outlyer.plugin_helper',
    description = 'Outlyer plugin helper',
    version='0.0.2',
    license='MIT',
    url='https://github.com/outlyerapp/plugin_helper',
    download_url='https://github.com/outlyerapp/plugin_helper/archive/0.1.tar.gz',

    author='Colin Hemmings',
    author_email='colin.hemmings@outlyer.com',
    keywords=['outlyer', 'plugin', 'helper'],

    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    install_requires=[
        'docker==2.2.0',
        'requests==2.12.4'
    ],
    packages=[
        'outlyer',
        'outlyer.plugin_helper',
        'outlyer.plugin_helper.container'
    ],
)
if __name__ == "__main__":
    pass